<?php
/**
 * Maximum Products per User for WooCommerce - Core Class
 *
 * @version 2.4.1
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_MPPU_Core' ) ) :

class Alg_WC_MPPU_Core {

	/**
	 * Constructor.
	 *
	 * @version 2.3.0
	 * @since   1.0.0
	 * @todo    [dev] maybe `is_admin && ! ajax`
	 * @todo    [dev] product terms (back-end): add fields to "add category/tag" pages also (i.e. not only to "edit" pages)
	 * @todo    [feature] zero quantity for "Guest"
	 * @todo    [feature] correct on add to cart
	 * @todo    [feature] limit max quantity with WC filter
	 */
	function __construct() {
		$this->is_wc_version_below_3_0_0 = version_compare( get_option( 'woocommerce_version', null ), '3.0.0', '<' );
		if ( 'yes' === get_option( 'wpjup_wc_maximum_products_per_user_plugin_enabled', 'yes' ) ) {
			// Properties
			$this->do_use_user_roles = ( 'yes' === get_option( 'alg_wc_mppu_use_user_roles', 'no' ) );
			// Check quantities
			add_action( 'woocommerce_checkout_process', array( $this, 'check_cart_quantities' ), PHP_INT_MAX );
			add_action( 'woocommerce_before_cart',      array( $this, 'check_cart_quantities' ), PHP_INT_MAX );
			// Block checkout page
			if ( 'yes' === get_option( 'wpjup_wc_maximum_products_per_user_stop_from_seeing_checkout', 'no' ) ) {
				add_action( 'wp', array( $this, 'block_checkout' ), PHP_INT_MAX );
			}
			// Validate on add to cart
			if ( 'yes' === get_option( 'alg_wc_mppu_validate_on_add_to_cart', 'yes' ) ) {
				add_filter( 'woocommerce_add_to_cart_validation', array( $this, 'validate_on_add_to_cart' ), PHP_INT_MAX, 4 );
			}
			// Shortcode
			add_shortcode( 'alg_wc_mppu_translate', array( $this, 'language_shortcode' ) );
			// Per product options
			require_once( 'settings/class-alg-wc-mppu-settings-per-product.php' );
			// Product terms options
			require_once( 'settings/class-alg-wc-mppu-settings-per-term.php' );
			// Update quantities
			require_once( 'class-alg-wc-mppu-data.php' );
			// Sales data reports
			require_once( 'class-alg-wc-mppu-reports.php' );
			// Users
			require_once( 'class-alg-wc-mppu-users.php' );
		}
		do_action( 'alg_wc_mppu_core_loaded', $this );
	}

	/**
	 * language_shortcode.
	 *
	 * @version 2.0.0
	 * @since   2.0.0
	 */
	function language_shortcode( $atts, $content = '' ) {
		// E.g.: `[alg_wc_mppu_translate lang="DE" lang_text="Message in German" not_lang_text="Message for all other languages"]`
		if ( isset( $atts['lang_text'] ) && isset( $atts['not_lang_text'] ) && ! empty( $atts['lang'] ) ) {
			return ( ! defined( 'ICL_LANGUAGE_CODE' ) || ! in_array( strtolower( ICL_LANGUAGE_CODE ), array_map( 'trim', explode( ',', strtolower( $atts['lang'] ) ) ) ) ) ?
				$atts['not_lang_text'] : $atts['lang_text'];
		}
		// E.g.: `[alg_wc_mppu_translate lang="DE"]Message in German[/alg_wc_mppu_translate][alg_wc_mppu_translate not_lang="DE"]Message for all other languages[/alg_wc_mppu_translate]`
		return (
			( ! empty( $atts['lang'] )     && ( ! defined( 'ICL_LANGUAGE_CODE' ) || ! in_array( strtolower( ICL_LANGUAGE_CODE ), array_map( 'trim', explode( ',', strtolower( $atts['lang'] ) ) ) ) ) ) ||
			( ! empty( $atts['not_lang'] ) &&     defined( 'ICL_LANGUAGE_CODE' ) &&   in_array( strtolower( ICL_LANGUAGE_CODE ), array_map( 'trim', explode( ',', strtolower( $atts['not_lang'] ) ) ) ) )
		) ? '' : $content;
	}

	/**
	 * handle_user_roles.
	 *
	 * @version 2.2.0
	 * @since   2.2.0
	 */
	function handle_user_roles( $role ) {
		return ( '' == $role ? 'guest' : ( 'super_admin' == $role ? 'administrator' : $role ) );
	}

	/**
	 * get_max_qty_for_user_role.
	 *
	 * @version 2.2.0
	 * @since   2.2.0
	 * @todo    [feature] (maybe) per user (now can be done via "Max Qty by Formula" section)
	 */
	function get_max_qty_for_user_role( $type, $product_or_term_id = 0 ) {
		// Get qty data
		switch ( $type ) {
			case 'per_product':
				$user_roles_data = ( '' != ( $data = get_post_meta( $product_or_term_id, '_alg_wc_mppu_user_roles_max_qty', true ) ) ? $data : array() );
				break;
			case 'all_products':
				$user_roles_data = get_option( 'alg_wc_mppu_user_roles_max_qty', array() );
				break;
			case 'per_term':
				$user_roles_data = ( '' != ( $data = get_term_meta( $product_or_term_id, '_alg_wc_mppu_user_roles_max_qty', true ) ) ? $data : array() );
				break;
		}
		// Get current user roles
		$current_user = wp_get_current_user();
		if ( ! isset( $current_user->roles ) || empty( $current_user->roles ) ) {
			$current_user->roles = array( 'guest' );
		}
		$current_user->roles = array_map( array( $this, 'handle_user_roles' ), $current_user->roles );
		// Return result
		foreach ( $current_user->roles as $role ) {
			if ( ! empty( $user_roles_data[ $role ] ) ) {
				return $user_roles_data[ $role ];
			}
		}
		// Nothing found for the user role
		return 0;
	}

	/**
	 * get_max_qty.
	 *
	 * @version 2.3.0
	 * @since   1.0.0
	 * @todo    [feature] `per_product`: add "enabled/disabled" option
	 * @todo    [feature] `all_products`: apply only to selected products (i.e. include/exclude products, cats, tags)
	 */
	function get_max_qty( $type, $product_or_term_id = 0 ) {
		if ( 0 != ( $max_qty_by_formula = apply_filters( 'alg_wc_mppu_max_qty_by_formula', 0, $type, $product_or_term_id ) ) ) {
			return apply_filters( 'alg_wc_mppu_get_max_qty', $max_qty_by_formula, $product_or_term_id, $type );
		}
		if ( $this->do_use_user_roles && 0 != ( $max_qty_for_user_role = $this->get_max_qty_for_user_role( $type, $product_or_term_id ) ) ) {
			return apply_filters( 'alg_wc_mppu_get_max_qty', $max_qty_for_user_role, $product_or_term_id, $type );
		}
		switch ( $type ) {
			case 'per_product':
				return apply_filters( 'alg_wc_mppu_get_max_qty', ( ( $qty = get_post_meta( $product_or_term_id, '_wpjup_wc_maximum_products_per_user_qty', true ) ) ? $qty : 0 ), $product_or_term_id, $type );
			case 'all_products':
				return apply_filters( 'alg_wc_mppu_get_max_qty', get_option( 'wpjup_wc_maximum_products_per_user_global_max_qty', 1 ), $product_or_term_id, $type );
			case 'per_term':
				return apply_filters( 'alg_wc_mppu_get_max_qty', ( ( $qty = get_term_meta( $product_or_term_id, '_alg_wc_mppu_qty', true ) ) ? $qty : 0 ), $product_or_term_id, $type );
		}
	}

	/**
	 * block_checkout.
	 *
	 * @version 2.2.0
	 * @since   1.0.0
	 */
	function block_checkout() {
		if ( ! function_exists( 'is_checkout' ) || ! is_checkout() ) {
			return;
		}
		if ( ! $this->check_quantities( false ) ) {
			wp_safe_redirect( wc_get_cart_url() );
			exit;
		}
	}

	/**
	 * validate_on_add_to_cart.
	 *
	 * @version 2.0.0
	 * @since   2.0.0
	 * @todo    [dev] (maybe) different message (i.e. different from "cart" message)
	 */
	function validate_on_add_to_cart( $passed, $product_id, $quantity, $variation_id = 0 ) {
		if ( 0 != ( $current_user_id = get_current_user_id() ) ) {
			$cart_item_quantities = WC()->cart->get_cart_item_quantities();
			if ( 0 != $variation_id ) {
				$product_id = $variation_id;
			}
			if ( ! empty( $cart_item_quantities[ $product_id ] ) ) {
				$quantity += $cart_item_quantities[ $product_id ];
			}
			if ( ! $this->check_quantities_for_product( $product_id, $quantity, false, true, $current_user_id, $cart_item_quantities ) ) {
				return false;
			}
		}
		return $passed;
	}

	/**
	 * check_cart_quantities.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 */
	function check_cart_quantities() {
		$this->check_quantities( true );
	}

	/**
	 * get_date_to_check.
	 *
	 * @version 2.4.0
	 * @since   2.4.0
	 * @todo    [dev] maybe add more predefined ranges, e.g. `last_45_days`, `last_60_days`, `MINUTE_IN_SECONDS`
	 */
	function get_date_to_check( $date_range ) {
		$current_time = ( 'time' === get_option( 'alg_wc_mppu_time_func', 'time' ) ? time() : ( int ) current_time( 'timestamp' ) );
		switch ( $date_range ) {
			case 'this_hour':
				$date_to_check = strtotime( date( 'Y-m-d H:00:00' ), $current_time );
				break;
			case 'this_day':
				$date_to_check = strtotime( date( 'Y-m-d 00:00:00' ), $current_time );
				break;
			case 'this_week':
				$date_to_check = strtotime( 'monday this week', $current_time );
				break;
			case 'this_month':
				$date_to_check = strtotime( date( 'Y-m-01' ), $current_time );
				break;
			case 'this_year':
				$date_to_check = strtotime( date( 'Y-01-01' ), $current_time );
				break;
			case 'last_hour':
				$date_to_check = ( $current_time - HOUR_IN_SECONDS );
				break;
			case 'last_24_hours':
				$date_to_check = ( $current_time - DAY_IN_SECONDS );
				break;
			case 'last_7_days':
				$date_to_check = ( $current_time - WEEK_IN_SECONDS );
				break;
			case 'last_30_days':
				$date_to_check = ( $current_time - MONTH_IN_SECONDS );
				break;
			case 'last_365_days':
				$date_to_check = ( $current_time - YEAR_IN_SECONDS );
				break;
			case 'custom':
				$date_to_check = ( $current_time - get_option( 'alg_wc_mppu_date_range_custom', 1 ) );
				break;
		}
		return apply_filters( 'alg_wc_mppu_date_to_check', $date_to_check, $date_range, $current_time );
	}

	/**
	 * check_order_date_range.
	 *
	 * @version 2.4.0
	 * @since   2.0.0
	 */
	function check_order_date_range( $order_date, $date_range ) {
		$date_to_check = $this->get_date_to_check( $date_range );
		return ( $order_date >= $date_to_check );
	}

	/**
	 * get_date_range.
	 *
	 * @version 2.1.0
	 * @since   2.1.0
	 * @todo    [feature] `alg_wc_mppu_date_range_per_product_or_term_id`
	 */
	function get_date_range( $product_or_term_id, $is_product ) {
		$date_range = get_option( 'alg_wc_mppu_date_range', 'lifetime' );
		/**
		 * TODO: Date range per product / term ID.
		 *
		 * if ( 'yes' === get_option( 'alg_wc_mppu_date_range_per_product_or_term_id', 'no' ) ) {
		 *     $date_range_per_product_or_term = ( $is_product ?
		 *         get_post_meta( $product_or_term_id, '_alg_wc_mppu_date_range', true ) :
		 *         get_term_meta( $product_or_term_id, '_alg_wc_mppu_date_range', true ) );
		 *     if ( ! empty( $date_range_per_product_or_term ) && 'default' != $date_range_per_product_or_term ) {
		 *         $date_range = $date_range_per_product_or_term;
		 *     }
		 * }
		 */
		return apply_filters( 'alg_wc_mppu_date_range', $date_range, $product_or_term_id, $is_product );
		/**
		 * For example: Custom date range for product ID 100.
		 *
		 * As of v2.1.0, possible date range values are:
		 * 'lifetime', 'this_hour', 'this_day', 'this_week', 'this_month', 'this_year', 'last_hour', 'last_24_hours', 'last_7_days', 'last_30_days', 'last_365_days'
		 *
		 * add_filter( 'alg_wc_mppu_date_range', 'my_custom_alg_wc_mppu_date_range', 10, 3 );
		 * function my_custom_alg_wc_mppu_date_range( $date_range, $product_or_term_id, $is_product ) {
		 *     return ( $is_product && 100 == $product_or_term_id ? 'lifetime' : $date_range );
		 * }
		 */
	}

	/**
	 * get_user_already_bought_qty.
	 *
	 * @version 2.1.0
	 * @since   2.0.0
	 * @todo    [dev] maybe add option to use e.g. order date completed (instead of `date_created`)
	 */
	function get_user_already_bought_qty( $product_or_term_id, $current_user_id, $is_product ) {
		$user_already_bought = 0;
		$date_range          = $this->get_date_range( $product_or_term_id, $is_product );
		if ( 'lifetime' === $date_range ) {
			$users_quantities = ( $is_product ?
				get_post_meta( $product_or_term_id, '_alg_wc_mppu_totals_data', true ) :
				get_term_meta( $product_or_term_id, '_alg_wc_mppu_totals_data', true ) );
			if ( $users_quantities && isset( $users_quantities[ $current_user_id ] ) ) {
				$user_already_bought = $users_quantities[ $current_user_id ];
			}
		} else {
			$users_quantities = ( $is_product ?
				get_post_meta( $product_or_term_id, '_alg_wc_mppu_orders_data', true ) :
				get_term_meta( $product_or_term_id, '_alg_wc_mppu_orders_data', true ) );
			if ( $users_quantities && isset( $users_quantities[ $current_user_id ] ) ) {
				foreach ( $users_quantities[ $current_user_id ] as $order_id => $order_data ) {
					if ( $this->check_order_date_range( $order_data['date_created'], $date_range ) ) {
						$user_already_bought += $order_data['qty'];
					}
				}
			}
		}
		return $user_already_bought;
	}

	/**
	 * output_notice.
	 *
	 * @version 2.4.1
	 * @since   2.0.0
	 * @todo    [dev] customizable notice type
	 * @todo    [dev] add more placeholders, e.g. `%qty_in_cart%` etc.
	 */
	function output_notice( $_product_id, $max_qty, $user_already_bought, $is_cart ) {
		$product         = wc_get_product( $_product_id );
		$replaced_values = array(
			'%max_qty%'             => ( $max_qty > 0 ? $max_qty : 0 ),
			'%product_title%'       => $product->get_name(),
			'%qty_already_bought%'  => $user_already_bought,
			'%remaining_qty%'       => max( ( $max_qty - $user_already_bought ), 0 ),
		);
		$message = get_option( 'wpjup_wc_maximum_products_per_user_message',
			__( 'You can only buy maximum %max_qty% pcs. of %product_title% (you already bought %qty_already_bought% pcs.).', 'maximum-products-per-user-for-woocommerce' ) );
		$message = do_shortcode( str_replace( array_keys( $replaced_values ), $replaced_values, $message ) );
		if ( $is_cart ) {
			wc_print_notice( $message, 'notice' );
		} else {
			wc_add_notice( $message, 'error' );
		}
	}

	/**
	 * get_cart_item_quantities_by_term.
	 *
	 * @version 2.0.0
	 * @since   2.0.0
	 */
	function get_cart_item_quantities_by_term( $current_product_id, $current_cart_item_quantity, $cart_item_quantities, $current_term_id, $taxonomy ) {
		$result = $current_cart_item_quantity;
		if ( ! empty( $cart_item_quantities ) ) {
			foreach ( $cart_item_quantities as $product_id => $product_qty ) {
				if ( $product_id != $current_product_id ) {
					if ( 0 != ( $parent_product_id = wp_get_post_parent_id( $product_id ) ) ) {
						$product_id = $parent_product_id;
					}
					$terms = get_the_terms( $product_id, $taxonomy );
					if ( $terms && ! is_wp_error( $terms ) ) {
						foreach( $terms as $term ) {
							if ( $term->term_id === $current_term_id ) {
								$result += $product_qty;
								break;
							}
						}
					}
				}
			}
		}
		return $result;
	}

	/**
	 * check_quantities_for_product.
	 *
	 * @version 2.0.0
	 * @since   2.0.0
	 */
	function check_quantities_for_product( $_product_id, $cart_item_quantity, $is_cart, $do_add_notices, $current_user_id, $cart_item_quantities ) {
		$parent_product_id = $this->get_product_id_or_variation_parent_id( wc_get_product( $_product_id ) );
		$product_id        = ( 'yes' === get_option( 'alg_wc_mppu_use_variations', 'no' ) ? $_product_id : $parent_product_id );
		// Maybe exclude products
		$exclude_products = get_option( 'alg_wc_mppu_exclude_products', array() );
		if ( ! empty( $exclude_products ) && in_array( $product_id, $exclude_products ) ) {
			return true;
		}
		if ( 'yes' === apply_filters( 'alg_wc_mppu_option', 'no', 'local_enabled' ) && 0 != ( $max_qty = $this->get_max_qty( 'per_product', $product_id ) ) ) {
			// Per product
			$user_already_bought = $this->get_user_already_bought_qty( $product_id, $current_user_id, true );
			if ( ( $user_already_bought + $cart_item_quantity ) > $max_qty ) {
				if ( $do_add_notices ) {
					$this->output_notice( $product_id, $max_qty, $user_already_bought, $is_cart );
				}
				return false;
			}
		} else {
			// Per taxonomy
			$validated_by_term = false;
			foreach ( array( 'product_cat', 'product_tag' ) as $taxonomy ) {
				if ( ! $validated_by_term && 'yes' === apply_filters( 'alg_wc_mppu_option', 'no', $taxonomy . '_enabled' ) ) {
					$terms = get_the_terms( $parent_product_id, $taxonomy );
					if ( $terms && ! is_wp_error( $terms ) ) {
						foreach( $terms as $term ) {
							if ( 0 != ( $max_qty = $this->get_max_qty( 'per_term', $term->term_id ) ) ) {
								$validated_by_term      = true;
								$cart_item_quantity_all = $this->get_cart_item_quantities_by_term( $_product_id, $cart_item_quantity, $cart_item_quantities, $term->term_id, $taxonomy );
								$user_already_bought    = $this->get_user_already_bought_qty( $term->term_id, $current_user_id, false );
								if ( ( $user_already_bought + $cart_item_quantity_all ) > $max_qty ) {
									if ( $do_add_notices ) {
										$this->output_notice( $_product_id, $max_qty, $user_already_bought, $is_cart );
									}
									return false;
								}
							}
						}
					}
				}
			}
			// All products
			if ( ! $validated_by_term && 'yes' === get_option( 'wpjup_wc_maximum_products_per_user_global_enabled', 'no' ) && 0 != ( $max_qty = $this->get_max_qty( 'all_products' ) ) ) {
				$user_already_bought = $this->get_user_already_bought_qty( $product_id, $current_user_id, true );
				if ( ( $user_already_bought + $cart_item_quantity ) > $max_qty ) {
					if ( $do_add_notices ) {
						$this->output_notice( $product_id, $max_qty, $user_already_bought, $is_cart );
					}
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * check_quantities.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 */
	function check_quantities( $do_add_notices ) {
		$result = true;
		if ( ! isset( WC()->cart ) ) {
			return $result;
		}
		if ( 0 == ( $current_user_id = get_current_user_id() ) ) {
			return $result;
		}
		$cart_item_quantities = WC()->cart->get_cart_item_quantities();
		if ( empty( $cart_item_quantities ) || ! is_array( $cart_item_quantities ) ) {
			return $result;
		}
		$is_cart = ( function_exists( 'is_cart' ) && is_cart() );
		foreach ( $cart_item_quantities as $_product_id => $cart_item_quantity ) {
			if ( ! $this->check_quantities_for_product( $_product_id, $cart_item_quantity, $is_cart, $do_add_notices, $current_user_id, $cart_item_quantities ) ) {
				if ( $do_add_notices ) {
					$result = false;
				} else {
					return false;
				}
			}
		}
		return $result;
	}

	/**
	 * get_product_id_or_variation_parent_id.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] (maybe) WPML - parent product ID
	 */
	function get_product_id_or_variation_parent_id( $_product ) {
		if ( ! $_product || ! is_object( $_product ) ) {
			return 0;
		}
		if ( $this->is_wc_version_below_3_0_0 ) {
			return $_product->id;
		} else {
			return ( $_product->is_type( 'variation' ) ) ? $_product->get_parent_id() : $_product->get_id();
		}
	}

	/**
	 * get_product_id.
	 *
	 * @version 2.0.0
	 * @since   2.0.0
	 * @todo    [dev] (maybe) WPML - parent product ID
	 */
	function get_product_id( $_product ) {
		if ( ! $_product || ! is_object( $_product ) ) {
			return 0;
		}
		if ( $this->is_wc_version_below_3_0_0 ) {
			return ( isset( $_product->variation_id ) ) ? $_product->variation_id : $_product->id;
		} else {
			return $_product->get_id();
		}
	}

}

endif;

return new Alg_WC_MPPU_Core();
