<?php
/**
 * Maximum Products per User for WooCommerce - Per Product Settings
 *
 * @version 2.4.0
 * @since   2.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_MPPU_Settings_Per_product' ) ) :

class Alg_WC_MPPU_Settings_Per_product {

	/**
	 * Constructor.
	 *
	 * @version 2.0.0
	 * @since   2.0.0
	 */
	function __construct() {
		if ( 'yes' === apply_filters( 'alg_wc_mppu_option', 'no', 'local_enabled' ) ) {
			add_action( 'add_meta_boxes',    array( $this, 'add_meta_box' ) );
			add_action( 'save_post_product', array( $this, 'save_meta_box' ), PHP_INT_MAX, 2 );
		}
	}

	/**
	 * add_meta_box.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 */
	function add_meta_box() {
		add_meta_box(
			'alg-wc-maximum-products-per-user',
			__( 'Maximum Products per User', 'maximum-products-per-user-for-woocommerce' ),
			array( $this, 'create_meta_box' ),
			'product',
			'normal',
			'high'
		);
	}

	/**
	 * create_meta_box.
	 *
	 * @version 2.2.0
	 * @since   1.0.0
	 */
	function create_meta_box() {
		$product_id = get_the_ID();
		$html = '';
		foreach ( $this->get_meta_box_options( $product_id ) as $option ) {
			$value = get_post_meta( $option['product_id'], $option['meta'], true );
			if ( isset( $option['key'] ) ) {
				$value = ( isset( $value[ $option['key'] ] ) ? $value[ $option['key'] ] : $option['default'] );
			}
			if ( '' === $value ) {
				$value = $option['default'];
			}
			$custom_attributes = ( isset( $option['custom_attributes'] ) ? ' ' . $option['custom_attributes'] : '' );
			$id                = $option['name'] . ( isset( $option['key'] ) ? '_' . $option['key']       : '' );
			$name              = $option['name'] . ( isset( $option['key'] ) ? '[' . $option['key'] . ']' : '' );
			$html .= '<tr>';
			$html .= '<td><label for="' . $id . '">' . $option['title'] . '</label></td>' . ' ' .
				'<td><input type="' . $option['type'] . '" id="' . $id . '" name="' . $name . '" value="' . $value . '"' . $custom_attributes . '>';
			if ( isset( $option['tooltip'] ) ) {
				$html .= wc_help_tip( $option['tooltip'], true );
			}
			$html .= '</td></tr>';
		}
		echo ( ! empty( $html ) ? '<table>' . $html . '</table>' : $html );
	}

	/**
	 * save_meta_box.
	 *
	 * @version 2.2.0
	 * @since   1.0.0
	 */
	function save_meta_box( $product_id, $__post ) {
		$data = array();
		foreach ( $this->get_meta_box_options( $product_id ) as $option ) {
			if ( isset( $_POST[ $option['name'] ] ) ) {
				if ( ! isset( $option['key'] ) ) {
					update_post_meta( $option['product_id'], $option['meta'], sanitize_text_field( $_POST[ $option['name'] ] ) );
				} else {
					$data[ $option['product_id'] ][ $option['meta'] ][ $option['key'] ] = sanitize_text_field( $_POST[ $option['name'] ][ $option['key'] ] );
				}
			}
		}
		if ( ! empty( $data ) ) {
			foreach ( $data as $_product_id => $product_data ) {
				foreach ( $product_data as $meta => $values ) {
					update_post_meta( $_product_id, $meta, $values );
				}
			}
		}
	}

	/**
	 * get_meta_box_options.
	 *
	 * @version 2.4.0
	 * @since   1.0.0
	 */
	function get_meta_box_options( $product_id = 0 ) {
		$result = array();
		if ( 0 == $product_id ) {
			$product_id = get_the_ID();
		}
		if ( 'yes' === get_option( 'alg_wc_mppu_use_variations', 'no' ) && ( $product = wc_get_product() ) && ( $children = $product->get_children() ) ) {
			foreach ( $children as $child_id ) {
				$result[] = array(
					'title'             => __( 'Maximum quantity per user', 'maximum-products-per-user-for-woocommerce' ) . ' (#' . $child_id . ')',
					'name'              => 'wpjup_wc_maximum_products_per_user_qty_' . $child_id,
					'meta'              => '_wpjup_wc_maximum_products_per_user_qty',
					'default'           => 0,
					'type'              => 'number',
					'tooltip'           => __( 'If set to zero, and "All Products" section is enabled - global maximum quantity will be used; in case if "All Products" section is disabled - no maximum quantity will be used.', 'maximum-products-per-user-for-woocommerce' ),
					'custom_attributes' => 'min="0"',
					'product_id'        => $child_id,
				);
				// User roles
				if ( 'yes' === get_option( 'alg_wc_mppu_use_user_roles', 'no' ) ) {
					foreach ( alg_wc_mppu_get_all_user_roles() as $role => $role_name ) {
						$result[] = array(
							'title'             => __( 'Maximum quantity per user', 'maximum-products-per-user-for-woocommerce' ) . ' (#' . $child_id . ')' . ': ' . $role_name,
							'name'              => 'alg_wc_mppu_user_roles_max_qty_' . $child_id,
							'key'               => $role,
							'meta'              => '_alg_wc_mppu_user_roles_max_qty',
							'default'           => 0,
							'type'              => 'number',
							'tooltip'           => __( 'If set to zero - "Maximum quantity per user" option will be used.', 'maximum-products-per-user-for-woocommerce' ),
							'custom_attributes' => 'min="0"',
							'product_id'        => $child_id,
						);
					}
				}
			}
		} else {
			$result[] = array(
				'title'             => __( 'Maximum quantity per user', 'maximum-products-per-user-for-woocommerce' ),
				'name'              => 'wpjup_wc_maximum_products_per_user_qty',
				'meta'              => '_wpjup_wc_maximum_products_per_user_qty',
				'default'           => 0,
				'type'              => 'number',
				'tooltip'           => __( 'If set to zero, and "All Products" section is enabled - global maximum quantity will be used; in case if "All Products" section is disabled - no maximum quantity will be used.', 'maximum-products-per-user-for-woocommerce' ),
				'custom_attributes' => 'min="0"',
				'product_id'        => $product_id,
			);
			// User roles
			if ( 'yes' === get_option( 'alg_wc_mppu_use_user_roles', 'no' ) ) {
				foreach ( alg_wc_mppu_get_all_user_roles() as $role => $role_name ) {
					$result[] = array(
						'title'             => __( 'Maximum quantity per user', 'maximum-products-per-user-for-woocommerce' ) . ': ' . $role_name,
						'name'              => 'alg_wc_mppu_user_roles_max_qty',
						'key'               => $role,
						'meta'              => '_alg_wc_mppu_user_roles_max_qty',
						'default'           => 0,
						'type'              => 'number',
						'tooltip'           => __( 'If set to zero - "Maximum quantity per user" option will be used.', 'maximum-products-per-user-for-woocommerce' ),
						'custom_attributes' => 'min="0"',
						'product_id'        => $product_id,
					);
				}
			}
		}
		return $result;
	}

}

endif;

return new Alg_WC_MPPU_Settings_Per_product();
